import java.lang
import org.ergoplatform
import org.ergoplatform.wallet.boxes
import org.ergoplatform.wallet.crypto
import org.ergoplatform.wallet.interface4j
import org.ergoplatform.wallet.interpreter
import org.ergoplatform.wallet.mnemonic
import org.ergoplatform.wallet.protocol
import org.ergoplatform.wallet.secrets
import org.ergoplatform.wallet.serialization
import org.ergoplatform.wallet.settings
import org.ergoplatform.wallet.transactions
import org.ergoplatform.wallet.utils
import scala
import scala.collection
import scala.collection.immutable
import scala.collection.mutable
import scala.math
import scala.util
import supertagged
import typing



class AssetUtils:
    @staticmethod
    def mergeAssets(initialMap: scala.collection.immutable.Map[typing.Union[java.lang.String, str], typing.Any], maps: scala.collection.Seq[scala.collection.immutable.Map[typing.Union[java.lang.String, str], typing.Any]]) -> scala.collection.immutable.Map[java.lang.String, typing.Any]: ...
    @staticmethod
    def mergeAssetsMut(into: scala.collection.mutable.Map[typing.Union[java.lang.String, str], typing.Any], from_: scala.collection.Seq[scala.collection.immutable.Map[typing.Union[java.lang.String, str], typing.Any]]) -> None: ...
    @staticmethod
    def subtractAssets(initialMap: scala.collection.immutable.Map[typing.Union[java.lang.String, str], typing.Any], subtractor: scala.collection.Seq[scala.collection.immutable.Map[typing.Union[java.lang.String, str], typing.Any]]) -> scala.collection.immutable.Map[java.lang.String, typing.Any]: ...
    @staticmethod
    def subtractAssetsMut(from_: scala.collection.mutable.Map[typing.Union[java.lang.String, str], typing.Any], subtractor: scala.collection.immutable.Map[typing.Union[java.lang.String, str], typing.Any]) -> None: ...

class Constants:
    @staticmethod
    def BitcoinSeed() -> typing.List[int]: ...
    @staticmethod
    def CoinType() -> int: ...
    @staticmethod
    def Encoding() -> java.lang.String: ...
    @staticmethod
    def MaxAssetsPerBox() -> int: ...
    @staticmethod
    def MiningScanId() -> int: ...
    @staticmethod
    def ModifierIdLength() -> int: ...
    @staticmethod
    def PaymentsScanId() -> int: ...
    @staticmethod
    def SecretKeyLength() -> int: ...
    @staticmethod
    def eip3DerivationPath() -> org.ergoplatform.wallet.secrets.DerivationPath: ...
    @staticmethod
    def preEip3DerivationPath() -> org.ergoplatform.wallet.secrets.DerivationPath: ...
    class ScanId$(supertagged.package.TaggedType[typing.Any]):
        MODULE$: typing.ClassVar['Constants.ScanId.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.Any, supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.Any, supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.Any, supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.Any]) -> scala.math.Ordering[typing.Any]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.Any, supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...

_Utils__EitherOpsFor211__A = typing.TypeVar('_Utils__EitherOpsFor211__A')  # <A>
_Utils__EitherOpsFor211__B = typing.TypeVar('_Utils__EitherOpsFor211__B')  # <B>
class Utils:
    @staticmethod
    def paymentTransaction(recipientAddress: org.ergoplatform.ErgoAddress, changeAddress: org.ergoplatform.ErgoAddress, transferAmt: int, feeAmt: int, changeAmt: int, inputIds: typing.List[java.lang.String], currentHeight: int) -> org.ergoplatform.UnsignedErgoLikeTransaction: ...
    class EitherOpsFor211(typing.Generic[_Utils__EitherOpsFor211__A, _Utils__EitherOpsFor211__B]):
        def __init__(self, source: scala.util.Either[_Utils__EitherOpsFor211__A, _Utils__EitherOpsFor211__B]): ...
        def equals(self, x$1: typing.Any) -> bool: ...
        _flatMapRight__A1 = typing.TypeVar('_flatMapRight__A1')  # <A1>
        _flatMapRight__B1 = typing.TypeVar('_flatMapRight__B1')  # <B1>
        def flatMapRight(self, f: scala.Function1[_Utils__EitherOpsFor211__B, scala.util.Either[_flatMapRight__A1, _flatMapRight__B1]]) -> scala.util.Either[_flatMapRight__A1, _flatMapRight__B1]: ...
        def hashCode(self) -> int: ...
        _mapRight__B1 = typing.TypeVar('_mapRight__B1')  # <B1>
        def mapRight(self, f: scala.Function1[_Utils__EitherOpsFor211__B, _mapRight__B1]) -> scala.util.Either[_Utils__EitherOpsFor211__A, _mapRight__B1]: ...
        def source(self) -> scala.util.Either[_Utils__EitherOpsFor211__A, _Utils__EitherOpsFor211__B]: ...
    class EitherOpsFor211$:
        MODULE$: typing.ClassVar['Utils.EitherOpsFor211.'] = ...
        def __init__(self): ...
        _equals$extension__A = typing.TypeVar('_equals$extension__A')  # <A>
        _equals$extension__B = typing.TypeVar('_equals$extension__B')  # <B>
        def equals$extension(self, $this: scala.util.Either[_equals.extension__A, _equals.extension__B], x$1: typing.Any) -> bool: ...
        _flatMapRight$extension__A1 = typing.TypeVar('_flatMapRight$extension__A1')  # <A1>
        _flatMapRight$extension__B1 = typing.TypeVar('_flatMapRight$extension__B1')  # <B1>
        _flatMapRight$extension__A = typing.TypeVar('_flatMapRight$extension__A')  # <A>
        _flatMapRight$extension__B = typing.TypeVar('_flatMapRight$extension__B')  # <B>
        def flatMapRight$extension(self, $this: scala.util.Either[_flatMapRight.extension__A, _flatMapRight.extension__B], f: scala.Function1[_flatMapRight.extension__B, scala.util.Either[_flatMapRight.extension__A1, _flatMapRight.extension__B1]]) -> scala.util.Either[_flatMapRight.extension__A1, _flatMapRight.extension__B1]: ...
        _hashCode$extension__A = typing.TypeVar('_hashCode$extension__A')  # <A>
        _hashCode$extension__B = typing.TypeVar('_hashCode$extension__B')  # <B>
        def hashCode$extension(self, $this: scala.util.Either[_hashCode.extension__A, _hashCode.extension__B]) -> int: ...
        _mapRight$extension__B1 = typing.TypeVar('_mapRight$extension__B1')  # <B1>
        _mapRight$extension__A = typing.TypeVar('_mapRight$extension__A')  # <A>
        _mapRight$extension__B = typing.TypeVar('_mapRight$extension__B')  # <B>
        def mapRight$extension(self, $this: scala.util.Either[_mapRight.extension__A, _mapRight.extension__B], f: scala.Function1[_mapRight.extension__B, _mapRight.extension__B1]) -> scala.util.Either[_mapRight.extension__A, _mapRight.extension__B1]: ...

class package: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("org.ergoplatform.wallet")``.

    AssetUtils: typing.Type[AssetUtils]
    Constants: typing.Type[Constants]
    Utils: typing.Type[Utils]
    package: typing.Type[package]
    boxes: org.ergoplatform.wallet.boxes.__module_protocol__
    crypto: org.ergoplatform.wallet.crypto.__module_protocol__
    interface4j: org.ergoplatform.wallet.interface4j.__module_protocol__
    interpreter: org.ergoplatform.wallet.interpreter.__module_protocol__
    mnemonic: org.ergoplatform.wallet.mnemonic.__module_protocol__
    protocol: org.ergoplatform.wallet.protocol.__module_protocol__
    secrets: org.ergoplatform.wallet.secrets.__module_protocol__
    serialization: org.ergoplatform.wallet.serialization.__module_protocol__
    settings: org.ergoplatform.wallet.settings.__module_protocol__
    transactions: org.ergoplatform.wallet.transactions.__module_protocol__
    utils: org.ergoplatform.wallet.utils.__module_protocol__



import io.circe
import java.lang
import org.bouncycastle.math.ec.custom.sec
import scala
import scala.collection
import scala.reflect
import scala.runtime
import scala.util
import scorex.util.serialization
import sigmastate
import sigmastate.serialization
import special.collection
import special.sigma
import typing



_Helpers__DecoderResultOps__A = typing.TypeVar('_Helpers__DecoderResultOps__A')  # <A>
_Helpers__EitherOps__A = typing.TypeVar('_Helpers__EitherOps__A')  # <A>
_Helpers__EitherOps__B = typing.TypeVar('_Helpers__EitherOps__B')  # <B>
_Helpers__TryOps__A = typing.TypeVar('_Helpers__TryOps__A')  # <A>
class Helpers:
    @staticmethod
    def allOf(arr: typing.List[bool]) -> bool: ...
    @staticmethod
    def anyOf(arr: typing.List[bool]) -> bool: ...
    @staticmethod
    def castArray(array: typing.Any, evidence$3: scala.reflect.ClassTag) -> typing.Any: ...
    @typing.overload
    @staticmethod
    def concatArrays(arr1: typing.Any, arr2: typing.Any, evidence$1: scala.reflect.ClassTag) -> typing.Any: ...
    @typing.overload
    @staticmethod
    def concatArrays(seq: scala.collection.Traversable, evidence$2: scala.reflect.ClassTag) -> typing.Any: ...
    @staticmethod
    def concatBytes(seq: scala.collection.Traversable[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def decodeBytes(base16String: typing.Union[java.lang.String, str]) -> special.collection.Coll[typing.Any]: ...
    @staticmethod
    def decodeECPoint(hexString: typing.Union[java.lang.String, str]) -> org.bouncycastle.math.ec.custom.sec.SecP256K1Point: ...
    @staticmethod
    def decodeGroupElement(hexString: typing.Union[java.lang.String, str]) -> special.sigma.GroupElement: ...
    _deepHashCode__T = typing.TypeVar('_deepHashCode__T')  # <T>
    @staticmethod
    def deepHashCode(arr: typing.Any) -> int: ...
    _optionArrayEquals__A = typing.TypeVar('_optionArrayEquals__A')  # <A>
    @staticmethod
    def optionArrayEquals(maybeA1: scala.Option[typing.Any], maybeA2: scala.Option[typing.Any]) -> bool: ...
    @typing.overload
    @staticmethod
    def xor(ba1: typing.List[int], ba2: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def xor(bas: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def xorU(ba1: typing.List[int], ba2: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def xorU(target: typing.List[int], xss: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    class DecoderResultOps(typing.Generic[_Helpers__DecoderResultOps__A]):
        def __init__(self, source: scala.util.Either[io.circe.DecodingFailure, _Helpers__DecoderResultOps__A]): ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def source(self) -> scala.util.Either[io.circe.DecodingFailure, _Helpers__DecoderResultOps__A]: ...
        def toTry(self) -> scala.util.Try[_Helpers__DecoderResultOps__A]: ...
    class DecoderResultOps$:
        MODULE$: typing.ClassVar['Helpers.DecoderResultOps.'] = ...
        def __init__(self): ...
        _equals$extension__A = typing.TypeVar('_equals$extension__A')  # <A>
        def equals$extension(self, $this: scala.util.Either[io.circe.DecodingFailure, _equals.extension__A], x$1: typing.Any) -> bool: ...
        _hashCode$extension__A = typing.TypeVar('_hashCode$extension__A')  # <A>
        def hashCode$extension(self, $this: scala.util.Either[io.circe.DecodingFailure, _hashCode.extension__A]) -> int: ...
        _toTry$extension__A = typing.TypeVar('_toTry$extension__A')  # <A>
        def toTry$extension(self, $this: scala.util.Either[io.circe.DecodingFailure, _toTry.extension__A]) -> scala.util.Try[_toTry.extension__A]: ...
    class EitherOps(typing.Generic[_Helpers__EitherOps__A, _Helpers__EitherOps__B]):
        def __init__(self, source: scala.util.Either[_Helpers__EitherOps__A, _Helpers__EitherOps__B]): ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        _mapRight__B1 = typing.TypeVar('_mapRight__B1')  # <B1>
        def mapRight(self, f: scala.Function1[_Helpers__EitherOps__B, _mapRight__B1]) -> scala.util.Either[_Helpers__EitherOps__A, _mapRight__B1]: ...
        def source(self) -> scala.util.Either[_Helpers__EitherOps__A, _Helpers__EitherOps__B]: ...
    class EitherOps$:
        MODULE$: typing.ClassVar['Helpers.EitherOps.'] = ...
        def __init__(self): ...
        _equals$extension__A = typing.TypeVar('_equals$extension__A')  # <A>
        _equals$extension__B = typing.TypeVar('_equals$extension__B')  # <B>
        def equals$extension(self, $this: scala.util.Either[_equals.extension__A, _equals.extension__B], x$1: typing.Any) -> bool: ...
        _hashCode$extension__A = typing.TypeVar('_hashCode$extension__A')  # <A>
        _hashCode$extension__B = typing.TypeVar('_hashCode$extension__B')  # <B>
        def hashCode$extension(self, $this: scala.util.Either[_hashCode.extension__A, _hashCode.extension__B]) -> int: ...
        _mapRight$extension__B1 = typing.TypeVar('_mapRight$extension__B1')  # <B1>
        _mapRight$extension__A = typing.TypeVar('_mapRight$extension__A')  # <A>
        _mapRight$extension__B = typing.TypeVar('_mapRight$extension__B')  # <B>
        def mapRight$extension(self, $this: scala.util.Either[_mapRight.extension__A, _mapRight.extension__B], f: scala.Function1[_mapRight.extension__B, _mapRight.extension__B1]) -> scala.util.Either[_mapRight.extension__A, _mapRight.extension__B1]: ...
    class TryOps(typing.Generic[_Helpers__TryOps__A]):
        def __init__(self, source: scala.util.Try[_Helpers__TryOps__A]): ...
        def equals(self, x$1: typing.Any) -> bool: ...
        _fold__B = typing.TypeVar('_fold__B')  # <B>
        def fold(self, onError: scala.Function1[java.lang.Throwable, _fold__B], onSuccess: scala.Function1[_Helpers__TryOps__A, _fold__B]) -> _fold__B: ...
        def getOrThrow(self) -> _Helpers__TryOps__A: ...
        def hashCode(self) -> int: ...
        _mapOrThrow__B = typing.TypeVar('_mapOrThrow__B')  # <B>
        def mapOrThrow(self, f: scala.Function1[_Helpers__TryOps__A, _mapOrThrow__B]) -> _mapOrThrow__B: ...
        def source(self) -> scala.util.Try[_Helpers__TryOps__A]: ...
        def toEither(self) -> scala.util.Either[java.lang.Throwable, _Helpers__TryOps__A]: ...
    class TryOps$:
        MODULE$: typing.ClassVar['Helpers.TryOps.'] = ...
        def __init__(self): ...
        _equals$extension__A = typing.TypeVar('_equals$extension__A')  # <A>
        def equals$extension(self, $this: scala.util.Try[_equals.extension__A], x$1: typing.Any) -> bool: ...
        _fold$extension__B = typing.TypeVar('_fold$extension__B')  # <B>
        _fold$extension__A = typing.TypeVar('_fold$extension__A')  # <A>
        def fold$extension(self, $this: scala.util.Try[_fold.extension__A], onError: scala.Function1[java.lang.Throwable, _fold.extension__B], onSuccess: scala.Function1[_fold.extension__A, _fold.extension__B]) -> _fold.extension__B: ...
        _getOrThrow$extension__A = typing.TypeVar('_getOrThrow$extension__A')  # <A>
        def getOrThrow$extension(self, $this: scala.util.Try[_getOrThrow.extension__A]) -> _getOrThrow.extension__A: ...
        _hashCode$extension__A = typing.TypeVar('_hashCode$extension__A')  # <A>
        def hashCode$extension(self, $this: scala.util.Try[_hashCode.extension__A]) -> int: ...
        _mapOrThrow$extension__B = typing.TypeVar('_mapOrThrow$extension__B')  # <B>
        _mapOrThrow$extension__A = typing.TypeVar('_mapOrThrow$extension__A')  # <A>
        def mapOrThrow$extension(self, $this: scala.util.Try[_mapOrThrow.extension__A], f: scala.Function1[_mapOrThrow.extension__A, _mapOrThrow.extension__B]) -> _mapOrThrow.extension__B: ...
        _toEither$extension__A = typing.TypeVar('_toEither$extension__A')  # <A>
        def toEither$extension(self, $this: scala.util.Try[_toEither.extension__A]) -> scala.util.Either[java.lang.Throwable, _toEither.extension__A]: ...

class Overloading:
    @staticmethod
    def overload1() -> 'Overloading.Overload1': ...
    @staticmethod
    def overload2() -> 'Overloading.Overload2': ...
    @staticmethod
    def overload3() -> 'Overloading.Overload3': ...
    class Overload1:
        def __init__(self): ...
    class Overload2:
        def __init__(self): ...
    class Overload3:
        def __init__(self): ...

class SigmaByteReader(scorex.util.serialization.Reader):
    def __init__(self, r: scorex.util.serialization.Reader, constantStore: sigmastate.serialization.ConstantStore, resolvePlaceholdersToConstants: bool, maxTreeDepth: int): ...
    @staticmethod
    def $lessinit$greater$default$4() -> int: ...
    def addComplexity(self, delta: int) -> None: ...
    def complexity(self) -> int: ...
    def complexity_$eq(self, v: int) -> None: ...
    def constantStore(self) -> sigmastate.serialization.ConstantStore: ...
    def constantStore_$eq(self, x$1: sigmastate.serialization.ConstantStore) -> None: ...
    def consumed(self) -> int: ...
    def getAllBufferBytes(self) -> typing.List[int]: ...
    def getBits(self, size: int) -> typing.List[bool]: ...
    def getByte(self) -> int: ...
    def getBytes(self, size: int) -> typing.List[int]: ...
    def getBytesUnsafe(self, numRequestedBytes: int) -> typing.List[int]: ...
    def getChunk(self, size: int) -> typing.Any: ...
    def getInt(self) -> int: ...
    def getLong(self) -> int: ...
    _getOption__T = typing.TypeVar('_getOption__T')  # <T>
    def getOption(self, getValue: scala.Function0[_getOption__T]) -> scala.Option[_getOption__T]: ...
    def getShort(self) -> int: ...
    def getShortString(self) -> java.lang.String: ...
    def getType(self) -> sigmastate.SType: ...
    def getUByte(self) -> int: ...
    def getUInt(self) -> int: ...
    def getULong(self) -> int: ...
    def getUShort(self) -> int: ...
    def getValue(self) -> sigmastate.Values.Value[sigmastate.SType]: ...
    def getValues(self) -> scala.collection.IndexedSeq[sigmastate.Values.Value[sigmastate.SType]]: ...
    def level(self) -> int: ...
    def level_$eq(self, v: int) -> None: ...
    def mark(self) -> 'SigmaByteReader': ...
    def maxTreeDepth(self) -> int: ...
    def newReader(self, chunk: typing.Any) -> scorex.util.serialization.Reader: ...
    def peekByte(self) -> int: ...
    def position(self) -> int: ...
    def positionLimit(self) -> int: ...
    def positionLimit_$eq(self, v: int) -> None: ...
    def position_$eq(self, p: int) -> None: ...
    def r(self) -> scorex.util.serialization.Reader: ...
    def remaining(self) -> int: ...
    def resolvePlaceholdersToConstants(self) -> bool: ...
    def resolvePlaceholdersToConstants_$eq(self, x$1: bool) -> None: ...
    def sigmastate$utils$SigmaByteReader$$_complexity(self) -> int: ...
    def sigmastate$utils$SigmaByteReader$$_complexity_$eq(self, x$1: int) -> None: ...
    def sigmastate$utils$SigmaByteReader$$_wasDeserialize(self) -> bool: ...
    def sigmastate$utils$SigmaByteReader$$_wasDeserialize_$eq(self, x$1: bool) -> None: ...
    def sigmastate$utils$SigmaByteReader$$checkPositionLimit(self) -> None: ...
    def sigmastate$utils$SigmaByteReader$$lvl(self) -> int: ...
    def sigmastate$utils$SigmaByteReader$$lvl_$eq(self, x$1: int) -> None: ...
    def sigmastate$utils$SigmaByteReader$$positionLmt(self) -> int: ...
    def sigmastate$utils$SigmaByteReader$$positionLmt_$eq(self, x$1: int) -> None: ...
    def valDefTypeStore(self) -> sigmastate.serialization.ValDefTypeStore: ...
    def wasDeserialize(self) -> bool: ...
    def wasDeserialize_$eq(self, v: bool) -> None: ...

_SparseArrayContainer__T = typing.TypeVar('_SparseArrayContainer__T')  # <T>
class SparseArrayContainer(typing.Generic[_SparseArrayContainer__T]):
    def __init__(self, values: scala.collection.Seq[scala.Tuple2[typing.Any, _SparseArrayContainer__T]], evidence$1: scala.reflect.ClassTag[_SparseArrayContainer__T]): ...
    def add(self, code: int, value: _SparseArrayContainer__T) -> None: ...
    def apply(self, code: int) -> _SparseArrayContainer__T: ...
    @staticmethod
    def buildForSerializers(sers: scala.collection.Seq[sigmastate.serialization.ValueSerializer[sigmastate.Values.Value[sigmastate.SType]]]) -> 'SparseArrayContainer'[sigmastate.serialization.ValueSerializer[sigmastate.Values.Value[sigmastate.SType]]]: ...
    def get(self, code: int) -> scala.Option[_SparseArrayContainer__T]: ...
    def remove(self, code: int) -> None: ...
    def sigmastate$utils$SparseArrayContainer$$codeToIndex(self, code: int) -> int: ...
    def sigmastate$utils$SparseArrayContainer$$sparseArray(self) -> typing.Any: ...

_SigmaByteWriter__DataInfo__T = typing.TypeVar('_SigmaByteWriter__DataInfo__T')  # <T>
_SigmaByteWriter__FormatDescriptor__T = typing.TypeVar('_SigmaByteWriter__FormatDescriptor__T')  # <T>
_SigmaByteWriter__SeqFmt__T = typing.TypeVar('_SigmaByteWriter__SeqFmt__T')  # <T>
_SigmaByteWriter__U__T = typing.TypeVar('_SigmaByteWriter__U__T')  # <T>
_SigmaByteWriter__UVlqFmt__T = typing.TypeVar('_SigmaByteWriter__UVlqFmt__T')  # <T>
_SigmaByteWriter__Vlq__T = typing.TypeVar('_SigmaByteWriter__Vlq__T')  # <T>
_SigmaByteWriter__ZigZag__T = typing.TypeVar('_SigmaByteWriter__ZigZag__T')  # <T>
_SigmaByteWriter__ZigZagFmt__T = typing.TypeVar('_SigmaByteWriter__ZigZagFmt__T')  # <T>
_SigmaByteWriter__ZigZagVlqFmt__T = typing.TypeVar('_SigmaByteWriter__ZigZagVlqFmt__T')  # <T>
class SigmaByteWriter(scorex.util.serialization.Writer):
    def __init__(self, w: scorex.util.serialization.Writer, constantExtractionStore: scala.Option[sigmastate.serialization.ConstantStore]): ...
    _argInfoToDataInfo__T = typing.TypeVar('_argInfoToDataInfo__T')  # <T>
    @staticmethod
    def argInfoToDataInfo(arg: sigmastate.ArgInfo, fmt: 'SigmaByteWriter.FormatDescriptor'[_argInfoToDataInfo__T]) -> 'SigmaByteWriter.DataInfo'[_argInfoToDataInfo__T]: ...
    @staticmethod
    def bitsInfo(name: typing.Union[java.lang.String, str], desc: typing.Union[java.lang.String, str]) -> 'SigmaByteWriter.DataInfo'['SigmaByteWriter.Bits']: ...
    @staticmethod
    def bitsInfo$default$2() -> java.lang.String: ...
    def constantExtractionStore(self) -> scala.Option[sigmastate.serialization.ConstantStore]: ...
    def length(self) -> int: ...
    @staticmethod
    def maxBitsInfo(name: typing.Union[java.lang.String, str], maxBits: int, desc: typing.Union[java.lang.String, str]) -> 'SigmaByteWriter.DataInfo'['SigmaByteWriter.Bits']: ...
    @staticmethod
    def maxBitsInfo$default$3() -> java.lang.String: ...
    _nameToDataInfo__T = typing.TypeVar('_nameToDataInfo__T')  # <T>
    @staticmethod
    def nameToDataInfo(name: typing.Union[java.lang.String, str], fmt: 'SigmaByteWriter.FormatDescriptor'[_nameToDataInfo__T]) -> 'SigmaByteWriter.DataInfo'[_nameToDataInfo__T]: ...
    def newWriter(self) -> scorex.util.serialization.Writer: ...
    @typing.overload
    def put(self, x: int) -> 'SigmaByteWriter': ...
    @typing.overload
    def put(self, x: int, info: 'SigmaByteWriter.DataInfo'[typing.Any]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putBits(self, xs: typing.List[bool]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putBits(self, xs: typing.List[bool], info: 'SigmaByteWriter.DataInfo'['SigmaByteWriter.Bits']) -> 'SigmaByteWriter': ...
    @typing.overload
    def putBoolean(self, x: bool) -> 'SigmaByteWriter': ...
    @typing.overload
    def putBoolean(self, x: bool, info: 'SigmaByteWriter.DataInfo'[typing.Any]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putBytes(self, xs: typing.List[int]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putBytes(self, xs: typing.List[int], offset: int, length: int) -> 'SigmaByteWriter': ...
    @typing.overload
    def putBytes(self, xs: typing.List[int], info: 'SigmaByteWriter.DataInfo'[typing.List[int]]) -> 'SigmaByteWriter': ...
    def putChunk(self, chunk: typing.Any) -> 'SigmaByteWriter': ...
    @typing.overload
    def putInt(self, x: int) -> 'SigmaByteWriter': ...
    @typing.overload
    def putInt(self, x: int, info: 'SigmaByteWriter.DataInfo'[typing.Any]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putLong(self, x: int) -> 'SigmaByteWriter': ...
    @typing.overload
    def putLong(self, x: int, info: 'SigmaByteWriter.DataInfo'['SigmaByteWriter.Vlq'['SigmaByteWriter.ZigZag'[typing.Any]]]) -> 'SigmaByteWriter': ...
    _putOption__T = typing.TypeVar('_putOption__T')  # <T>
    def putOption(self, x: scala.Option[_putOption__T], putValueC: scala.Function2['SigmaByteWriter', _putOption__T, scala.runtime.BoxedUnit]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putShort(self, x: int) -> 'SigmaByteWriter': ...
    @typing.overload
    def putShort(self, x: int, info: 'SigmaByteWriter.DataInfo'[typing.Any]) -> 'SigmaByteWriter': ...
    def putShortBytes(self, value: int) -> 'SigmaByteWriter': ...
    def putShortString(self, s: typing.Union[java.lang.String, str]) -> 'SigmaByteWriter': ...
    _putType_0__T = typing.TypeVar('_putType_0__T', bound=sigmastate.SType)  # <T>
    _putType_1__T = typing.TypeVar('_putType_1__T', bound=sigmastate.SType)  # <T>
    @typing.overload
    def putType(self, x: _putType_0__T) -> 'SigmaByteWriter': ...
    @typing.overload
    def putType(self, x: _putType_1__T, info: 'SigmaByteWriter.DataInfo'[sigmastate.SType]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putUByte(self, x: int) -> 'SigmaByteWriter': ...
    @typing.overload
    def putUByte(self, x: int, info: 'SigmaByteWriter.DataInfo'['SigmaByteWriter.U'[typing.Any]]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putUInt(self, x: int) -> 'SigmaByteWriter': ...
    @typing.overload
    def putUInt(self, x: int, info: 'SigmaByteWriter.DataInfo'['SigmaByteWriter.Vlq'['SigmaByteWriter.U'[typing.Any]]]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putULong(self, x: int) -> 'SigmaByteWriter': ...
    @typing.overload
    def putULong(self, x: int, info: 'SigmaByteWriter.DataInfo'['SigmaByteWriter.Vlq'['SigmaByteWriter.U'[typing.Any]]]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putUShort(self, x: int) -> 'SigmaByteWriter': ...
    @typing.overload
    def putUShort(self, x: int, info: 'SigmaByteWriter.DataInfo'['SigmaByteWriter.Vlq'['SigmaByteWriter.U'[typing.Any]]]) -> 'SigmaByteWriter': ...
    _putValue_0__T = typing.TypeVar('_putValue_0__T', bound=sigmastate.SType)  # <T>
    _putValue_1__T = typing.TypeVar('_putValue_1__T', bound=sigmastate.SType)  # <T>
    @typing.overload
    def putValue(self, x: sigmastate.Values.Value[_putValue_0__T]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putValue(self, x: sigmastate.Values.Value[_putValue_1__T], info: 'SigmaByteWriter.DataInfo'[sigmastate.Values.Value[sigmastate.SType]]) -> 'SigmaByteWriter': ...
    _putValues_0__T = typing.TypeVar('_putValues_0__T', bound=sigmastate.SType)  # <T>
    _putValues_1__T = typing.TypeVar('_putValues_1__T', bound=sigmastate.SType)  # <T>
    @typing.overload
    def putValues(self, xs: scala.collection.Seq[sigmastate.Values.Value[_putValues_0__T]]) -> 'SigmaByteWriter': ...
    @typing.overload
    def putValues(self, xs: scala.collection.Seq[sigmastate.Values.Value[_putValues_1__T]], info: 'SigmaByteWriter.DataInfo'[scala.collection.Seq[sigmastate.Values.Value[sigmastate.SType]]], itemInfo: 'SigmaByteWriter.DataInfo'[sigmastate.Values.Value[sigmastate.SType]]) -> 'SigmaByteWriter': ...
    def result(self) -> typing.Any: ...
    def toBytes(self) -> typing.List[int]: ...
    _toSeqFmt__T = typing.TypeVar('_toSeqFmt__T')  # <T>
    @staticmethod
    def toSeqFmt(fmt: 'SigmaByteWriter.FormatDescriptor'[_toSeqFmt__T]) -> 'SigmaByteWriter.FormatDescriptor'[scala.collection.Seq[_toSeqFmt__T]]: ...
    _toUVlqFmt__T = typing.TypeVar('_toUVlqFmt__T')  # <T>
    @staticmethod
    def toUVlqFmt(fmt: 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.U'[_toUVlqFmt__T]]) -> 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.Vlq'['SigmaByteWriter.U'[_toUVlqFmt__T]]]: ...
    _toZigZagFmt__T = typing.TypeVar('_toZigZagFmt__T')  # <T>
    @staticmethod
    def toZigZagFmt(fmt: 'SigmaByteWriter.FormatDescriptor'[_toZigZagFmt__T]) -> 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.ZigZag'[_toZigZagFmt__T]]: ...
    _toZigZagVlqFmt__T = typing.TypeVar('_toZigZagVlqFmt__T')  # <T>
    @staticmethod
    def toZigZagVlqFmt(fmt: 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.ZigZag'[_toZigZagVlqFmt__T]]) -> 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.Vlq'['SigmaByteWriter.ZigZag'[_toZigZagVlqFmt__T]]]: ...
    @staticmethod
    def valuesItemInfo(info: 'SigmaByteWriter.DataInfo'[scala.collection.Seq[sigmastate.Values.Value[sigmastate.SType]]]) -> 'SigmaByteWriter.DataInfo'[sigmastate.Values.Value[sigmastate.SType]]: ...
    @staticmethod
    def valuesLengthInfo() -> 'SigmaByteWriter.DataInfo'['SigmaByteWriter.Vlq'['SigmaByteWriter.U'[typing.Any]]]: ...
    def w(self) -> scorex.util.serialization.Writer: ...
    class Bits: ...
    class BitsFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor['SigmaByteWriter.Bits'], scala.Product, scala.Serializable):
        MODULE$: typing.ClassVar['SigmaByteWriter.BitsFmt.'] = ...
        def __init__(self): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class BooleanFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor[typing.Any]):
        MODULE$: typing.ClassVar['SigmaByteWriter.BooleanFmt.'] = ...
        def __init__(self): ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class ByteFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor[typing.Any]):
        MODULE$: typing.ClassVar['SigmaByteWriter.ByteFmt.'] = ...
        def __init__(self): ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class DataInfo(scala.Product, scala.Serializable, typing.Generic[_SigmaByteWriter__DataInfo__T]):
        def __init__(self, info: sigmastate.ArgInfo, format: 'SigmaByteWriter.FormatDescriptor'[_SigmaByteWriter__DataInfo__T]): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        _copy__T = typing.TypeVar('_copy__T')  # <T>
        def copy(self, info: sigmastate.ArgInfo, format: 'SigmaByteWriter.FormatDescriptor'[typing.Any]) -> 'SigmaByteWriter.DataInfo'[typing.Any]: ...
        _copy$default$1__T = typing.TypeVar('_copy$default$1__T')  # <T>
        def copy$default$1(self) -> sigmastate.ArgInfo: ...
        _copy$default$2__T = typing.TypeVar('_copy$default$2__T')  # <T>
        def copy$default$2(self) -> 'SigmaByteWriter.FormatDescriptor'[typing.Any]: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def format(self) -> 'SigmaByteWriter.FormatDescriptor'[_SigmaByteWriter__DataInfo__T]: ...
        def hashCode(self) -> int: ...
        def info(self) -> sigmastate.ArgInfo: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class DataInfo$(scala.Serializable):
        MODULE$: typing.ClassVar['SigmaByteWriter.DataInfo.'] = ...
        def __init__(self): ...
        _apply__T = typing.TypeVar('_apply__T')  # <T>
        def apply(self, info: sigmastate.ArgInfo, format: 'SigmaByteWriter.FormatDescriptor'[_apply__T]) -> 'SigmaByteWriter.DataInfo'[_apply__T]: ...
        def toString(self) -> java.lang.String: ...
        _unapply__T = typing.TypeVar('_unapply__T')  # <T>
        def unapply(self, x$0: 'SigmaByteWriter.DataInfo'[_unapply__T]) -> scala.Option[scala.Tuple2[sigmastate.ArgInfo, 'SigmaByteWriter.FormatDescriptor'[_unapply__T]]]: ...
    class FormatDescriptor(typing.Generic[_SigmaByteWriter__FormatDescriptor__T]):
        def size(self) -> java.lang.String: ...
    class IntFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor[typing.Any]):
        MODULE$: typing.ClassVar['SigmaByteWriter.IntFmt.'] = ...
        def __init__(self): ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class LongFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor[typing.Any]):
        MODULE$: typing.ClassVar['SigmaByteWriter.LongFmt.'] = ...
        def __init__(self): ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class MaxBitsFmt(sigmastate.utils.SigmaByteWriter.FormatDescriptor['SigmaByteWriter.Bits'], scala.Product, scala.Serializable):
        def __init__(self, maxBits: int): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        def copy(self, maxBits: int) -> 'SigmaByteWriter.MaxBitsFmt': ...
        def copy$default$1(self) -> int: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def maxBits(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class MaxBitsFmt$(scala.runtime.AbstractFunction1[typing.Any, 'SigmaByteWriter.MaxBitsFmt'], scala.Serializable):
        MODULE$: typing.ClassVar['SigmaByteWriter.MaxBitsFmt.'] = ...
        def __init__(self): ...
        def apply(self, maxBits: int) -> 'SigmaByteWriter.MaxBitsFmt': ...
        def toString(self) -> java.lang.String: ...
        def unapply(self, x$0: 'SigmaByteWriter.MaxBitsFmt') -> scala.Option[typing.Any]: ...
    class SeqFmt(sigmastate.utils.SigmaByteWriter.FormatDescriptor[scala.collection.Seq[_SigmaByteWriter__SeqFmt__T]], scala.Product, scala.Serializable, typing.Generic[_SigmaByteWriter__SeqFmt__T]):
        def __init__(self, fmt: 'SigmaByteWriter.FormatDescriptor'[_SigmaByteWriter__SeqFmt__T]): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        _copy__T = typing.TypeVar('_copy__T')  # <T>
        def copy(self, fmt: 'SigmaByteWriter.FormatDescriptor'[typing.Any]) -> 'SigmaByteWriter.SeqFmt'[typing.Any]: ...
        _copy$default$1__T = typing.TypeVar('_copy$default$1__T')  # <T>
        def copy$default$1(self) -> 'SigmaByteWriter.FormatDescriptor'[typing.Any]: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def fmt(self) -> 'SigmaByteWriter.FormatDescriptor'[_SigmaByteWriter__SeqFmt__T]: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class SeqFmt$(scala.Serializable):
        MODULE$: typing.ClassVar['SigmaByteWriter.SeqFmt.'] = ...
        def __init__(self): ...
        _apply__T = typing.TypeVar('_apply__T')  # <T>
        def apply(self, fmt: 'SigmaByteWriter.FormatDescriptor'[_apply__T]) -> 'SigmaByteWriter.SeqFmt'[_apply__T]: ...
        def toString(self) -> java.lang.String: ...
        _unapply__T = typing.TypeVar('_unapply__T')  # <T>
        def unapply(self, x$0: 'SigmaByteWriter.SeqFmt'[_unapply__T]) -> scala.Option['SigmaByteWriter.FormatDescriptor'[_unapply__T]]: ...
    class ShortFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor[typing.Any]):
        MODULE$: typing.ClassVar['SigmaByteWriter.ShortFmt.'] = ...
        def __init__(self): ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class TypeFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor[sigmastate.SType], scala.Product, scala.Serializable):
        MODULE$: typing.ClassVar['SigmaByteWriter.TypeFmt.'] = ...
        def __init__(self): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class U(typing.Generic[_SigmaByteWriter__U__T]): ...
    class UByteFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor['SigmaByteWriter.U'[typing.Any]]):
        MODULE$: typing.ClassVar['SigmaByteWriter.UByteFmt.'] = ...
        def __init__(self): ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class UIntFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor['SigmaByteWriter.U'[typing.Any]]):
        MODULE$: typing.ClassVar['SigmaByteWriter.UIntFmt.'] = ...
        def __init__(self): ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class ULongFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor['SigmaByteWriter.U'[typing.Any]]):
        MODULE$: typing.ClassVar['SigmaByteWriter.ULongFmt.'] = ...
        def __init__(self): ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class UShortFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor['SigmaByteWriter.U'[typing.Any]]):
        MODULE$: typing.ClassVar['SigmaByteWriter.UShortFmt.'] = ...
        def __init__(self): ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class UVlqFmt(sigmastate.utils.SigmaByteWriter.FormatDescriptor['SigmaByteWriter.Vlq'['SigmaByteWriter.U'[_SigmaByteWriter__UVlqFmt__T]]], scala.Product, scala.Serializable, typing.Generic[_SigmaByteWriter__UVlqFmt__T]):
        def __init__(self, fmt: 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.U'[_SigmaByteWriter__UVlqFmt__T]]): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        _copy__T = typing.TypeVar('_copy__T')  # <T>
        def copy(self, fmt: 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.U'[typing.Any]]) -> 'SigmaByteWriter.UVlqFmt'[typing.Any]: ...
        _copy$default$1__T = typing.TypeVar('_copy$default$1__T')  # <T>
        def copy$default$1(self) -> 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.U'[typing.Any]]: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def fmt(self) -> 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.U'[_SigmaByteWriter__UVlqFmt__T]]: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class UVlqFmt$(scala.Serializable):
        MODULE$: typing.ClassVar['SigmaByteWriter.UVlqFmt.'] = ...
        def __init__(self): ...
        _apply__T = typing.TypeVar('_apply__T')  # <T>
        def apply(self, fmt: 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.U'[_apply__T]]) -> 'SigmaByteWriter.UVlqFmt'[_apply__T]: ...
        def toString(self) -> java.lang.String: ...
        _unapply__T = typing.TypeVar('_unapply__T')  # <T>
        def unapply(self, x$0: 'SigmaByteWriter.UVlqFmt'[_unapply__T]) -> scala.Option['SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.U'[_unapply__T]]]: ...
    class ValueFmt$(sigmastate.utils.SigmaByteWriter.FormatDescriptor[sigmastate.Values.Value[sigmastate.SType]], scala.Product, scala.Serializable):
        MODULE$: typing.ClassVar['SigmaByteWriter.ValueFmt.'] = ...
        def __init__(self): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class Vlq(typing.Generic[_SigmaByteWriter__Vlq__T]): ...
    class ZigZag(typing.Generic[_SigmaByteWriter__ZigZag__T]): ...
    class ZigZagFmt(sigmastate.utils.SigmaByteWriter.FormatDescriptor['SigmaByteWriter.ZigZag'[_SigmaByteWriter__ZigZagFmt__T]], scala.Product, scala.Serializable, typing.Generic[_SigmaByteWriter__ZigZagFmt__T]):
        def __init__(self, fmt: 'SigmaByteWriter.FormatDescriptor'[_SigmaByteWriter__ZigZagFmt__T]): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        _copy__T = typing.TypeVar('_copy__T')  # <T>
        def copy(self, fmt: 'SigmaByteWriter.FormatDescriptor'[typing.Any]) -> 'SigmaByteWriter.ZigZagFmt'[typing.Any]: ...
        _copy$default$1__T = typing.TypeVar('_copy$default$1__T')  # <T>
        def copy$default$1(self) -> 'SigmaByteWriter.FormatDescriptor'[typing.Any]: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def fmt(self) -> 'SigmaByteWriter.FormatDescriptor'[_SigmaByteWriter__ZigZagFmt__T]: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class ZigZagFmt$(scala.Serializable):
        MODULE$: typing.ClassVar['SigmaByteWriter.ZigZagFmt.'] = ...
        def __init__(self): ...
        _apply__T = typing.TypeVar('_apply__T')  # <T>
        def apply(self, fmt: 'SigmaByteWriter.FormatDescriptor'[_apply__T]) -> 'SigmaByteWriter.ZigZagFmt'[_apply__T]: ...
        def toString(self) -> java.lang.String: ...
        _unapply__T = typing.TypeVar('_unapply__T')  # <T>
        def unapply(self, x$0: 'SigmaByteWriter.ZigZagFmt'[_unapply__T]) -> scala.Option['SigmaByteWriter.FormatDescriptor'[_unapply__T]]: ...
    class ZigZagVlqFmt(sigmastate.utils.SigmaByteWriter.FormatDescriptor['SigmaByteWriter.Vlq'['SigmaByteWriter.ZigZag'[_SigmaByteWriter__ZigZagVlqFmt__T]]], scala.Product, scala.Serializable, typing.Generic[_SigmaByteWriter__ZigZagVlqFmt__T]):
        def __init__(self, fmt: 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.ZigZag'[_SigmaByteWriter__ZigZagVlqFmt__T]]): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        _copy__T = typing.TypeVar('_copy__T')  # <T>
        def copy(self, fmt: 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.ZigZag'[typing.Any]]) -> 'SigmaByteWriter.ZigZagVlqFmt'[typing.Any]: ...
        _copy$default$1__T = typing.TypeVar('_copy$default$1__T')  # <T>
        def copy$default$1(self) -> 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.ZigZag'[typing.Any]]: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def fmt(self) -> 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.ZigZag'[_SigmaByteWriter__ZigZagVlqFmt__T]]: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> java.lang.String: ...
        def size(self) -> java.lang.String: ...
        def toString(self) -> java.lang.String: ...
    class ZigZagVlqFmt$(scala.Serializable):
        MODULE$: typing.ClassVar['SigmaByteWriter.ZigZagVlqFmt.'] = ...
        def __init__(self): ...
        _apply__T = typing.TypeVar('_apply__T')  # <T>
        def apply(self, fmt: 'SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.ZigZag'[_apply__T]]) -> 'SigmaByteWriter.ZigZagVlqFmt'[_apply__T]: ...
        def toString(self) -> java.lang.String: ...
        _unapply__T = typing.TypeVar('_unapply__T')  # <T>
        def unapply(self, x$0: 'SigmaByteWriter.ZigZagVlqFmt'[_unapply__T]) -> scala.Option['SigmaByteWriter.FormatDescriptor'['SigmaByteWriter.ZigZag'[_unapply__T]]]: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("sigmastate.utils")``.

    Helpers: typing.Type[Helpers]
    Overloading: typing.Type[Overloading]
    SigmaByteReader: typing.Type[SigmaByteReader]
    SigmaByteWriter: typing.Type[SigmaByteWriter]
    SparseArrayContainer: typing.Type[SparseArrayContainer]



import java.io
import java.lang
import java.time
import java.time.chrono
import java.time.format
import java.util
import java.util.function
import typing



class IsoFields:
    DAY_OF_QUARTER: typing.ClassVar['TemporalField'] = ...
    QUARTER_OF_YEAR: typing.ClassVar['TemporalField'] = ...
    WEEK_OF_WEEK_BASED_YEAR: typing.ClassVar['TemporalField'] = ...
    WEEK_BASED_YEAR: typing.ClassVar['TemporalField'] = ...
    WEEK_BASED_YEARS: typing.ClassVar['TemporalUnit'] = ...
    QUARTER_YEARS: typing.ClassVar['TemporalUnit'] = ...

class JulianFields:
    JULIAN_DAY: typing.ClassVar['TemporalField'] = ...
    MODIFIED_JULIAN_DAY: typing.ClassVar['TemporalField'] = ...
    RATA_DIE: typing.ClassVar['TemporalField'] = ...

class TemporalAccessor:
    def get(self, temporalField: 'TemporalField') -> int: ...
    def getLong(self, temporalField: 'TemporalField') -> int: ...
    def isSupported(self, temporalField: 'TemporalField') -> bool: ...
    _query__R = typing.TypeVar('_query__R')  # <R>
    def query(self, temporalQuery: typing.Union['TemporalQuery'[_query__R], typing.Callable[['TemporalAccessor'], _query__R]]) -> _query__R: ...
    def range(self, temporalField: 'TemporalField') -> 'ValueRange': ...

class TemporalAdjuster:
    def adjustInto(self, temporal: 'Temporal') -> 'Temporal': ...

class TemporalAdjusters:
    @staticmethod
    def dayOfWeekInMonth(int: int, dayOfWeek: java.time.DayOfWeek) -> TemporalAdjuster: ...
    @staticmethod
    def firstDayOfMonth() -> TemporalAdjuster: ...
    @staticmethod
    def firstDayOfNextMonth() -> TemporalAdjuster: ...
    @staticmethod
    def firstDayOfNextYear() -> TemporalAdjuster: ...
    @staticmethod
    def firstDayOfYear() -> TemporalAdjuster: ...
    @staticmethod
    def firstInMonth(dayOfWeek: java.time.DayOfWeek) -> TemporalAdjuster: ...
    @staticmethod
    def lastDayOfMonth() -> TemporalAdjuster: ...
    @staticmethod
    def lastDayOfYear() -> TemporalAdjuster: ...
    @staticmethod
    def lastInMonth(dayOfWeek: java.time.DayOfWeek) -> TemporalAdjuster: ...
    @staticmethod
    def next(dayOfWeek: java.time.DayOfWeek) -> TemporalAdjuster: ...
    @staticmethod
    def nextOrSame(dayOfWeek: java.time.DayOfWeek) -> TemporalAdjuster: ...
    @staticmethod
    def ofDateAdjuster(unaryOperator: typing.Union[java.util.function.UnaryOperator[java.time.LocalDate], typing.Callable]) -> TemporalAdjuster: ...
    @staticmethod
    def previous(dayOfWeek: java.time.DayOfWeek) -> TemporalAdjuster: ...
    @staticmethod
    def previousOrSame(dayOfWeek: java.time.DayOfWeek) -> TemporalAdjuster: ...

class TemporalAmount:
    def addTo(self, temporal: 'Temporal') -> 'Temporal': ...
    def get(self, temporalUnit: 'TemporalUnit') -> int: ...
    def getUnits(self) -> java.util.List['TemporalUnit']: ...
    def subtractFrom(self, temporal: 'Temporal') -> 'Temporal': ...

class TemporalField:
    _adjustInto__R = typing.TypeVar('_adjustInto__R', bound='Temporal')  # <R>
    def adjustInto(self, r: _adjustInto__R, long: int) -> _adjustInto__R: ...
    def getBaseUnit(self) -> 'TemporalUnit': ...
    def getDisplayName(self, locale: java.util.Locale) -> str: ...
    def getFrom(self, temporalAccessor: TemporalAccessor) -> int: ...
    def getRangeUnit(self) -> 'TemporalUnit': ...
    def isDateBased(self) -> bool: ...
    def isSupportedBy(self, temporalAccessor: TemporalAccessor) -> bool: ...
    def isTimeBased(self) -> bool: ...
    def range(self) -> 'ValueRange': ...
    def rangeRefinedBy(self, temporalAccessor: TemporalAccessor) -> 'ValueRange': ...
    def resolve(self, map: typing.Union[java.util.Map['TemporalField', int], typing.Mapping['TemporalField', int]], temporalAccessor: TemporalAccessor, resolverStyle: java.time.format.ResolverStyle) -> TemporalAccessor: ...
    def toString(self) -> str: ...

class TemporalQueries:
    @staticmethod
    def chronology() -> 'TemporalQuery'[java.time.chrono.Chronology]: ...
    @staticmethod
    def localDate() -> 'TemporalQuery'[java.time.LocalDate]: ...
    @staticmethod
    def localTime() -> 'TemporalQuery'[java.time.LocalTime]: ...
    @staticmethod
    def offset() -> 'TemporalQuery'[java.time.ZoneOffset]: ...
    @staticmethod
    def precision() -> 'TemporalQuery'['TemporalUnit']: ...
    @staticmethod
    def zone() -> 'TemporalQuery'[java.time.ZoneId]: ...
    @staticmethod
    def zoneId() -> 'TemporalQuery'[java.time.ZoneId]: ...

_TemporalQuery__R = typing.TypeVar('_TemporalQuery__R')  # <R>
class TemporalQuery(typing.Generic[_TemporalQuery__R]):
    def queryFrom(self, temporalAccessor: TemporalAccessor) -> _TemporalQuery__R: ...

class TemporalUnit:
    _addTo__R = typing.TypeVar('_addTo__R', bound='Temporal')  # <R>
    def addTo(self, r: _addTo__R, long: int) -> _addTo__R: ...
    def between(self, temporal: 'Temporal', temporal2: 'Temporal') -> int: ...
    def getDuration(self) -> java.time.Duration: ...
    def isDateBased(self) -> bool: ...
    def isDurationEstimated(self) -> bool: ...
    def isSupportedBy(self, temporal: 'Temporal') -> bool: ...
    def isTimeBased(self) -> bool: ...
    def toString(self) -> str: ...

class UnsupportedTemporalTypeException(java.time.DateTimeException):
    @typing.overload
    def __init__(self, string: str): ...
    @typing.overload
    def __init__(self, string: str, throwable: java.lang.Throwable): ...

class ValueRange(java.io.Serializable):
    def checkValidIntValue(self, long: int, temporalField: TemporalField) -> int: ...
    def checkValidValue(self, long: int, temporalField: TemporalField) -> int: ...
    def equals(self, object: typing.Any) -> bool: ...
    def getLargestMinimum(self) -> int: ...
    def getMaximum(self) -> int: ...
    def getMinimum(self) -> int: ...
    def getSmallestMaximum(self) -> int: ...
    def hashCode(self) -> int: ...
    def isFixed(self) -> bool: ...
    def isIntValue(self) -> bool: ...
    def isValidIntValue(self, long: int) -> bool: ...
    def isValidValue(self, long: int) -> bool: ...
    @typing.overload
    @staticmethod
    def of(long: int, long2: int) -> 'ValueRange': ...
    @typing.overload
    @staticmethod
    def of(long: int, long2: int, long3: int) -> 'ValueRange': ...
    @typing.overload
    @staticmethod
    def of(long: int, long2: int, long3: int, long4: int) -> 'ValueRange': ...
    def toString(self) -> str: ...

class WeekFields(java.io.Serializable):
    ISO: typing.ClassVar['WeekFields'] = ...
    SUNDAY_START: typing.ClassVar['WeekFields'] = ...
    WEEK_BASED_YEARS: typing.ClassVar[TemporalUnit] = ...
    def dayOfWeek(self) -> TemporalField: ...
    def equals(self, object: typing.Any) -> bool: ...
    def getFirstDayOfWeek(self) -> java.time.DayOfWeek: ...
    def getMinimalDaysInFirstWeek(self) -> int: ...
    def hashCode(self) -> int: ...
    @typing.overload
    @staticmethod
    def of(dayOfWeek: java.time.DayOfWeek, int: int) -> 'WeekFields': ...
    @typing.overload
    @staticmethod
    def of(locale: java.util.Locale) -> 'WeekFields': ...
    def toString(self) -> str: ...
    def weekBasedYear(self) -> TemporalField: ...
    def weekOfMonth(self) -> TemporalField: ...
    def weekOfWeekBasedYear(self) -> TemporalField: ...
    def weekOfYear(self) -> TemporalField: ...

class ChronoField(java.lang.Enum['ChronoField'], TemporalField):
    NANO_OF_SECOND: typing.ClassVar['ChronoField'] = ...
    NANO_OF_DAY: typing.ClassVar['ChronoField'] = ...
    MICRO_OF_SECOND: typing.ClassVar['ChronoField'] = ...
    MICRO_OF_DAY: typing.ClassVar['ChronoField'] = ...
    MILLI_OF_SECOND: typing.ClassVar['ChronoField'] = ...
    MILLI_OF_DAY: typing.ClassVar['ChronoField'] = ...
    SECOND_OF_MINUTE: typing.ClassVar['ChronoField'] = ...
    SECOND_OF_DAY: typing.ClassVar['ChronoField'] = ...
    MINUTE_OF_HOUR: typing.ClassVar['ChronoField'] = ...
    MINUTE_OF_DAY: typing.ClassVar['ChronoField'] = ...
    HOUR_OF_AMPM: typing.ClassVar['ChronoField'] = ...
    CLOCK_HOUR_OF_AMPM: typing.ClassVar['ChronoField'] = ...
    HOUR_OF_DAY: typing.ClassVar['ChronoField'] = ...
    CLOCK_HOUR_OF_DAY: typing.ClassVar['ChronoField'] = ...
    AMPM_OF_DAY: typing.ClassVar['ChronoField'] = ...
    DAY_OF_WEEK: typing.ClassVar['ChronoField'] = ...
    ALIGNED_DAY_OF_WEEK_IN_MONTH: typing.ClassVar['ChronoField'] = ...
    ALIGNED_DAY_OF_WEEK_IN_YEAR: typing.ClassVar['ChronoField'] = ...
    DAY_OF_MONTH: typing.ClassVar['ChronoField'] = ...
    DAY_OF_YEAR: typing.ClassVar['ChronoField'] = ...
    EPOCH_DAY: typing.ClassVar['ChronoField'] = ...
    ALIGNED_WEEK_OF_MONTH: typing.ClassVar['ChronoField'] = ...
    ALIGNED_WEEK_OF_YEAR: typing.ClassVar['ChronoField'] = ...
    MONTH_OF_YEAR: typing.ClassVar['ChronoField'] = ...
    PROLEPTIC_MONTH: typing.ClassVar['ChronoField'] = ...
    YEAR_OF_ERA: typing.ClassVar['ChronoField'] = ...
    YEAR: typing.ClassVar['ChronoField'] = ...
    ERA: typing.ClassVar['ChronoField'] = ...
    INSTANT_SECONDS: typing.ClassVar['ChronoField'] = ...
    OFFSET_SECONDS: typing.ClassVar['ChronoField'] = ...
    _adjustInto__R = typing.TypeVar('_adjustInto__R', bound='Temporal')  # <R>
    def adjustInto(self, r: _adjustInto__R, long: int) -> _adjustInto__R: ...
    def checkValidIntValue(self, long: int) -> int: ...
    def checkValidValue(self, long: int) -> int: ...
    def getBaseUnit(self) -> TemporalUnit: ...
    def getDisplayName(self, locale: java.util.Locale) -> str: ...
    def getFrom(self, temporalAccessor: TemporalAccessor) -> int: ...
    def getRangeUnit(self) -> TemporalUnit: ...
    def isDateBased(self) -> bool: ...
    def isSupportedBy(self, temporalAccessor: TemporalAccessor) -> bool: ...
    def isTimeBased(self) -> bool: ...
    def range(self) -> ValueRange: ...
    def rangeRefinedBy(self, temporalAccessor: TemporalAccessor) -> ValueRange: ...
    def toString(self) -> str: ...
    _valueOf_0__T = typing.TypeVar('_valueOf_0__T', bound=java.lang.Enum)  # <T>
    @typing.overload
    @staticmethod
    def valueOf(class_: typing.Type[_valueOf_0__T], string: str) -> _valueOf_0__T: ...
    @typing.overload
    @staticmethod
    def valueOf(string: str) -> 'ChronoField': ...
    @staticmethod
    def values() -> typing.List['ChronoField']: ...

class ChronoUnit(java.lang.Enum['ChronoUnit'], TemporalUnit):
    NANOS: typing.ClassVar['ChronoUnit'] = ...
    MICROS: typing.ClassVar['ChronoUnit'] = ...
    MILLIS: typing.ClassVar['ChronoUnit'] = ...
    SECONDS: typing.ClassVar['ChronoUnit'] = ...
    MINUTES: typing.ClassVar['ChronoUnit'] = ...
    HOURS: typing.ClassVar['ChronoUnit'] = ...
    HALF_DAYS: typing.ClassVar['ChronoUnit'] = ...
    DAYS: typing.ClassVar['ChronoUnit'] = ...
    WEEKS: typing.ClassVar['ChronoUnit'] = ...
    MONTHS: typing.ClassVar['ChronoUnit'] = ...
    YEARS: typing.ClassVar['ChronoUnit'] = ...
    DECADES: typing.ClassVar['ChronoUnit'] = ...
    CENTURIES: typing.ClassVar['ChronoUnit'] = ...
    MILLENNIA: typing.ClassVar['ChronoUnit'] = ...
    ERAS: typing.ClassVar['ChronoUnit'] = ...
    FOREVER: typing.ClassVar['ChronoUnit'] = ...
    _addTo__R = typing.TypeVar('_addTo__R', bound='Temporal')  # <R>
    def addTo(self, r: _addTo__R, long: int) -> _addTo__R: ...
    def between(self, temporal: 'Temporal', temporal2: 'Temporal') -> int: ...
    def getDuration(self) -> java.time.Duration: ...
    def isDateBased(self) -> bool: ...
    def isDurationEstimated(self) -> bool: ...
    def isSupportedBy(self, temporal: 'Temporal') -> bool: ...
    def isTimeBased(self) -> bool: ...
    def toString(self) -> str: ...
    _valueOf_0__T = typing.TypeVar('_valueOf_0__T', bound=java.lang.Enum)  # <T>
    @typing.overload
    @staticmethod
    def valueOf(class_: typing.Type[_valueOf_0__T], string: str) -> _valueOf_0__T: ...
    @typing.overload
    @staticmethod
    def valueOf(string: str) -> 'ChronoUnit': ...
    @staticmethod
    def values() -> typing.List['ChronoUnit']: ...

class Temporal(TemporalAccessor):
    @typing.overload
    def isSupported(self, temporalUnit: TemporalUnit) -> bool: ...
    @typing.overload
    def isSupported(self, temporalField: TemporalField) -> bool: ...
    @typing.overload
    def minus(self, temporalAmount: TemporalAmount) -> 'Temporal': ...
    @typing.overload
    def minus(self, long: int, temporalUnit: TemporalUnit) -> 'Temporal': ...
    @typing.overload
    def plus(self, long: int, temporalUnit: TemporalUnit) -> 'Temporal': ...
    @typing.overload
    def plus(self, temporalAmount: TemporalAmount) -> 'Temporal': ...
    def until(self, temporal: 'Temporal', temporalUnit: TemporalUnit) -> int: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("java.time.temporal")``.

    ChronoField: typing.Type[ChronoField]
    ChronoUnit: typing.Type[ChronoUnit]
    IsoFields: typing.Type[IsoFields]
    JulianFields: typing.Type[JulianFields]
    Temporal: typing.Type[Temporal]
    TemporalAccessor: typing.Type[TemporalAccessor]
    TemporalAdjuster: typing.Type[TemporalAdjuster]
    TemporalAdjusters: typing.Type[TemporalAdjusters]
    TemporalAmount: typing.Type[TemporalAmount]
    TemporalField: typing.Type[TemporalField]
    TemporalQueries: typing.Type[TemporalQueries]
    TemporalQuery: typing.Type[TemporalQuery]
    TemporalUnit: typing.Type[TemporalUnit]
    UnsupportedTemporalTypeException: typing.Type[UnsupportedTemporalTypeException]
    ValueRange: typing.Type[ValueRange]
    WeekFields: typing.Type[WeekFields]



import org.ergoplatform.mining.emission
import typing


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("org.ergoplatform.mining")``.

    emission: org.ergoplatform.mining.emission.__module_protocol__



import java.io
import java.lang
import java.nio
import java.nio.charset.spi
import java.util
import typing



class CharacterCodingException(java.io.IOException):
    def __init__(self): ...

class Charset(java.lang.Comparable['Charset']):
    def aliases(self) -> java.util.Set[str]: ...
    @staticmethod
    def availableCharsets() -> java.util.SortedMap[str, 'Charset']: ...
    def canEncode(self) -> bool: ...
    def compareTo(self, charset: 'Charset') -> int: ...
    def contains(self, charset: 'Charset') -> bool: ...
    def decode(self, byteBuffer: java.nio.ByteBuffer) -> java.nio.CharBuffer: ...
    @staticmethod
    def defaultCharset() -> 'Charset': ...
    @typing.overload
    def displayName(self) -> str: ...
    @typing.overload
    def displayName(self, locale: java.util.Locale) -> str: ...
    @typing.overload
    def encode(self, string: str) -> java.nio.ByteBuffer: ...
    @typing.overload
    def encode(self, charBuffer: java.nio.CharBuffer) -> java.nio.ByteBuffer: ...
    def equals(self, object: typing.Any) -> bool: ...
    @staticmethod
    def forName(string: str) -> 'Charset': ...
    def hashCode(self) -> int: ...
    def isRegistered(self) -> bool: ...
    @staticmethod
    def isSupported(string: str) -> bool: ...
    def name(self) -> str: ...
    def newDecoder(self) -> 'CharsetDecoder': ...
    def newEncoder(self) -> 'CharsetEncoder': ...
    def toString(self) -> str: ...

class CharsetDecoder:
    def averageCharsPerByte(self) -> float: ...
    def charset(self) -> Charset: ...
    @typing.overload
    def decode(self, byteBuffer: java.nio.ByteBuffer) -> java.nio.CharBuffer: ...
    @typing.overload
    def decode(self, byteBuffer: java.nio.ByteBuffer, charBuffer: java.nio.CharBuffer, boolean: bool) -> 'CoderResult': ...
    def detectedCharset(self) -> Charset: ...
    def flush(self, charBuffer: java.nio.CharBuffer) -> 'CoderResult': ...
    def isAutoDetecting(self) -> bool: ...
    def isCharsetDetected(self) -> bool: ...
    def malformedInputAction(self) -> 'CodingErrorAction': ...
    def maxCharsPerByte(self) -> float: ...
    def onMalformedInput(self, codingErrorAction: 'CodingErrorAction') -> 'CharsetDecoder': ...
    def onUnmappableCharacter(self, codingErrorAction: 'CodingErrorAction') -> 'CharsetDecoder': ...
    def replaceWith(self, string: str) -> 'CharsetDecoder': ...
    def replacement(self) -> str: ...
    def reset(self) -> 'CharsetDecoder': ...
    def unmappableCharacterAction(self) -> 'CodingErrorAction': ...

class CharsetEncoder:
    def averageBytesPerChar(self) -> float: ...
    @typing.overload
    def canEncode(self, char: str) -> bool: ...
    @typing.overload
    def canEncode(self, charSequence: typing.Union[java.lang.CharSequence, str]) -> bool: ...
    def charset(self) -> Charset: ...
    @typing.overload
    def encode(self, charBuffer: java.nio.CharBuffer) -> java.nio.ByteBuffer: ...
    @typing.overload
    def encode(self, charBuffer: java.nio.CharBuffer, byteBuffer: java.nio.ByteBuffer, boolean: bool) -> 'CoderResult': ...
    def flush(self, byteBuffer: java.nio.ByteBuffer) -> 'CoderResult': ...
    def isLegalReplacement(self, byteArray: typing.List[int]) -> bool: ...
    def malformedInputAction(self) -> 'CodingErrorAction': ...
    def maxBytesPerChar(self) -> float: ...
    def onMalformedInput(self, codingErrorAction: 'CodingErrorAction') -> 'CharsetEncoder': ...
    def onUnmappableCharacter(self, codingErrorAction: 'CodingErrorAction') -> 'CharsetEncoder': ...
    def replaceWith(self, byteArray: typing.List[int]) -> 'CharsetEncoder': ...
    def replacement(self) -> typing.List[int]: ...
    def reset(self) -> 'CharsetEncoder': ...
    def unmappableCharacterAction(self) -> 'CodingErrorAction': ...

class CoderMalfunctionError(java.lang.Error):
    def __init__(self, exception: java.lang.Exception): ...

class CoderResult:
    UNDERFLOW: typing.ClassVar['CoderResult'] = ...
    OVERFLOW: typing.ClassVar['CoderResult'] = ...
    def isError(self) -> bool: ...
    def isMalformed(self) -> bool: ...
    def isOverflow(self) -> bool: ...
    def isUnderflow(self) -> bool: ...
    def isUnmappable(self) -> bool: ...
    def length(self) -> int: ...
    @staticmethod
    def malformedForLength(int: int) -> 'CoderResult': ...
    def throwException(self) -> None: ...
    def toString(self) -> str: ...
    @staticmethod
    def unmappableForLength(int: int) -> 'CoderResult': ...

class CodingErrorAction:
    IGNORE: typing.ClassVar['CodingErrorAction'] = ...
    REPLACE: typing.ClassVar['CodingErrorAction'] = ...
    REPORT: typing.ClassVar['CodingErrorAction'] = ...
    def toString(self) -> str: ...

class IllegalCharsetNameException(java.lang.IllegalArgumentException):
    def __init__(self, string: str): ...
    def getCharsetName(self) -> str: ...

class StandardCharsets:
    US_ASCII: typing.ClassVar[Charset] = ...
    ISO_8859_1: typing.ClassVar[Charset] = ...
    UTF_8: typing.ClassVar[Charset] = ...
    UTF_16BE: typing.ClassVar[Charset] = ...
    UTF_16LE: typing.ClassVar[Charset] = ...
    UTF_16: typing.ClassVar[Charset] = ...

class UnsupportedCharsetException(java.lang.IllegalArgumentException):
    def __init__(self, string: str): ...
    def getCharsetName(self) -> str: ...

class MalformedInputException(CharacterCodingException):
    def __init__(self, int: int): ...
    def getInputLength(self) -> int: ...
    def getMessage(self) -> str: ...

class UnmappableCharacterException(CharacterCodingException):
    def __init__(self, int: int): ...
    def getInputLength(self) -> int: ...
    def getMessage(self) -> str: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("java.nio.charset")``.

    CharacterCodingException: typing.Type[CharacterCodingException]
    Charset: typing.Type[Charset]
    CharsetDecoder: typing.Type[CharsetDecoder]
    CharsetEncoder: typing.Type[CharsetEncoder]
    CoderMalfunctionError: typing.Type[CoderMalfunctionError]
    CoderResult: typing.Type[CoderResult]
    CodingErrorAction: typing.Type[CodingErrorAction]
    IllegalCharsetNameException: typing.Type[IllegalCharsetNameException]
    MalformedInputException: typing.Type[MalformedInputException]
    StandardCharsets: typing.Type[StandardCharsets]
    UnmappableCharacterException: typing.Type[UnmappableCharacterException]
    UnsupportedCharsetException: typing.Type[UnsupportedCharsetException]
    spi: java.nio.charset.spi.__module_protocol__



import scala
import scala.math
import supertagged
import typing



class Curve25519:
    @staticmethod
    def KeyLength() -> int: ...
    @staticmethod
    def KeyLength25519() -> int: ...
    @staticmethod
    def SignatureLength() -> int: ...
    @staticmethod
    def SignatureLength25519() -> int: ...
    @typing.overload
    @staticmethod
    def createKeyPair() -> scala.Tuple2[typing.List[int], typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def createKeyPair(seed: typing.List[int]) -> scala.Tuple2[typing.List[int], typing.List[int]]: ...
    @staticmethod
    def createSharedSecret(privateKey: typing.List[int], publicKey: typing.List[int]) -> typing.List[int]: ...
    @staticmethod
    def sign(privateKey: typing.List[int], message: typing.List[int]) -> typing.List[int]: ...
    @staticmethod
    def verify(signature: typing.List[int], message: typing.List[int], publicKey: typing.List[int]) -> bool: ...

class SigningFunctions:
    @staticmethod
    def $init$($this: 'SigningFunctions') -> None: ...
    def KeyLength(self) -> int: ...
    def SignatureLength(self) -> int: ...
    @typing.overload
    def createKeyPair(self, seed: typing.List[int]) -> scala.Tuple2[typing.List[int], typing.List[int]]: ...
    @typing.overload
    def createKeyPair(self) -> scala.Tuple2[typing.List[int], typing.List[int]]: ...
    def createSharedSecret(self, privateKey: typing.List[int], publicKey: typing.List[int]) -> typing.List[int]: ...
    def sign(self, privateKey: typing.List[int], message: typing.List[int]) -> typing.List[int]: ...
    def verify(self, signature: typing.List[int], message: typing.List[int], publicKey: typing.List[int]) -> bool: ...

class package:
    class PrivateKey$(supertagged.package.TaggedType[typing.List[int]]):
        MODULE$: typing.ClassVar['package.PrivateKey.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class PublicKey$(supertagged.package.TaggedType[typing.List[int]]):
        MODULE$: typing.ClassVar['package.PublicKey.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class SharedSecret$(supertagged.package.TaggedType[typing.List[int]]):
        MODULE$: typing.ClassVar['package.SharedSecret.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class Signature$(supertagged.package.TaggedType[typing.List[int]]):
        MODULE$: typing.ClassVar['package.Signature.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...

class EllipticCurveSignatureScheme(SigningFunctions): ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("scorex.crypto.signatures")``.

    Curve25519: typing.Type[Curve25519]
    EllipticCurveSignatureScheme: typing.Type[EllipticCurveSignatureScheme]
    SigningFunctions: typing.Type[SigningFunctions]
    package: typing.Type[package]
